@@ -350,12 +350,15 @@ static int tunnel_to(int sock, ip_type ip, unsigned short port, proxy_type pt, c
 
 				if(2 != read_n_bytes(sock, in, 2))
 					goto err;
-				if(in[0] != 1 || in[1] != 0) {
-					if(in[0] != 1)
-						goto err;
-					else
-						return BLOCKED;
-				}
+	/* according to RFC 1929 the version field for the user/pass auth sub-
+	   negotiation should be 1, which is kinda counter-intuitive, so there
+	   are some socks5 proxies that return 5 instead. other programs like
+	   curl work fine when the version is 5, so let's do the same and accept
+	   either of them. */
+				if(!(in[0] == 5 || in[0] == 1))
+					goto err;
+				if(in[1] != 0)
+					return BLOCKED;
 			}
 			int buff_iter = 0;
 			buff[buff_iter++] = 5;	// version
@@ -787,8 +790,8 @@ void proxy_freeaddrinfo(struct addrinfo *res) {
 	free(res);
 }
 
-#if defined(IS_MAC) || defined(IS_OPENBSD)
-#ifdef IS_OPENBSD /* OpenBSD has its own incompatible getservbyname_r */
+#if defined(IS_MAC) || defined(IS_OPENBSD) || defined(IS_SOLARIS)
+#if defined(IS_OPENBSD) || defined(IS_SOLARIS) /* OpenBSD and Solaris has its own incompatible getservbyname_r */
 #define getservbyname_r mygetservbyname_r
 #endif
 /* getservbyname on mac is using thread local storage, so we dont need mutex 
@@ -830,7 +833,10 @@ int proxy_getaddrinfo(const char *node, const char *service, const struct addrin
 	if(node && !inet_aton(node, &((struct sockaddr_in *) &space->sockaddr_space)->sin_addr)) {
 		/* some folks (nmap) use getaddrinfo() with AI_NUMERICHOST to check whether a string
 		   containing a numeric ip was passed. we must return failure in that case. */
-		if(hints && (hints->ai_flags & AI_NUMERICHOST)) return EAI_NONAME;
+		if(hints && (hints->ai_flags & AI_NUMERICHOST)) {
+			free(space);
+			return EAI_NONAME;
+		}
 		hp = proxy_gethostbyname(node, &ghdata);
 		if(hp)
 			memcpy(&((struct sockaddr_in *) &space->sockaddr_space)->sin_addr,
